#include "ftc_abi_safe.h"
#include "ftc.h"
#include <cstring>

namespace ftc_detail {

namespace {

struct implementation
{
    implementation (const char* id, bool writable, const char* host
                    , unsigned short port, const char* accessKey)
        : ftc_impl (id, std::strlen(id), writable, host, port, accessKey, 16) {}
    ftc ftc_impl;
};

implementation* get_impl (ftc_abi_safe::implementation* impl)
{
    return static_cast<implementation*>(static_cast<void*>(impl));
}

}

}

ftc_abi_safe::ftc_abi_safe(const char* id, bool writable, const char* host
                           , unsigned short port, const char* accessKey)
    : impl (0)
{
    ftc_detail::implementation* impl_
        = new ftc_detail::implementation (id, writable, host
                                          , port, accessKey);
    impl = static_cast<ftc_abi_safe::implementation*>(
        static_cast<void*>(impl_));
}

ftc_abi_safe::~ftc_abi_safe()
{
    delete ftc_detail::get_impl (impl);
}

void ftc_abi_safe::open( bool readonly )
{
    ftc_detail::get_impl (impl)->ftc_impl.open (readonly);
}

bool ftc_abi_safe::isOpen()
{
    return ftc_detail::get_impl (impl)->ftc_impl.isOpen ();
}

void ftc_abi_safe::close()
{
    ftc_detail::get_impl (impl)->ftc_impl.close ();
}

void ftc_abi_safe::setPosition( unsigned long long position )
{
    ftc_detail::get_impl (impl)->ftc_impl.setPosition (position);
}

unsigned long long ftc_abi_safe::getPosition() const
{
    return ftc_detail::get_impl (impl)->ftc_impl.getPosition ();
}

void ftc_abi_safe::setSize( unsigned long long size )
{
    ftc_detail::get_impl (impl)->ftc_impl.setSize (size);
}

unsigned long long ftc_abi_safe::getSize() const
{
    return ftc_detail::get_impl (impl)->ftc_impl.getSize ();
}

unsigned long long ftc_abi_safe::read(char* data, unsigned long long nbytes, unsigned long long position)
{
    return ftc_detail::get_impl (impl)->ftc_impl.read (data, nbytes, position);
}

unsigned long long ftc_abi_safe::write(const char* data, unsigned long long nbytes, unsigned long long position)
{
    return ftc_detail::get_impl (impl)->ftc_impl.write (data, nbytes, position);
}

unsigned long long ftc_abi_safe::transferTo(unsigned long long position, unsigned long long nbytes, FILE* fd, char* buffer)
{
    return ftc_detail::get_impl (impl)->ftc_impl.transferTo (position, nbytes, fd, buffer);
}
